@@ -4,32 +4,35 @@ import time
 from copy import deepcopy
 import warnings
 
-from django.utils.encoding import python_2_unicode_compatible
-from django.utils.six import text_type
 from django.db import models
 from django.template import Context, Template
 from django.core.mail import EmailMessage
 
+
 def _render_from_string(templatestring, context=None):
     if context is None:
         context = {}
     template = Template(templatestring)
     return template.render(Context(context))
 
+
 class MailrobotError(ValueError):
     pass
 
+
 class MailrobotNoSenderError(MailrobotError):
     pass
 
+
 class MailrobotNoRecipientsError(MailrobotError):
     pass
 
+
 class NameManager(models.Manager):
     def get_by_natural_key(self, name):
         return self.get(name=name)
 
-@python_2_unicode_compatible
+
 class AbstractNamedModel(models.Model):
     NAME_MAX_LENGTH = 40
     name = models.SlugField(max_length=NAME_MAX_LENGTH, unique=True)
@@ -61,11 +64,12 @@ class AbstractNamedModel(models.Model):
         newself.save(force_insert=True)
         return newself
 
+
 class AddressManager(models.Manager):
     def get_by_natural_key(self, address):
         return self.get(address=address)
 
-@python_2_unicode_compatible
+
 class Address(models.Model):
     address = models.EmailField(unique=True)
     comment = models.CharField(max_length=66, blank=True, null=True)
@@ -83,6 +87,7 @@ class Address(models.Model):
     def natural_key(self):
         return self.address
 
+
 class Signature(AbstractNamedModel):
     """Email signature
 
@@ -109,6 +114,7 @@ class Signature(AbstractNamedModel):
             return '\r\n\r\n\r\n-- \r\n%s' % signature
         return ''
 
+
 class MailBody(AbstractNamedModel):
     "Subject and bodytext of the email"
 
@@ -118,6 +124,7 @@ class MailBody(AbstractNamedModel):
     class Meta:
         verbose_name_plural = 'mailbodies'
 
+
 class Mail(AbstractNamedModel):
     """Canned Mail with default sender, Reply-To and recipients
 
@@ -129,13 +136,24 @@ class Mail(AbstractNamedModel):
         'blank': True,
     }
 
-    content = models.ForeignKey(MailBody, on_delete=models.CASCADE, related_name='mail')
-    signature = models.ForeignKey(Signature, related_name='mail', **KEYFIELD_DEFAULTS)
-    sender = models.ForeignKey(Address, related_name='sender', **KEYFIELD_DEFAULTS)
-    recipients = models.ManyToManyField(Address, related_name='recipients', **KEYFIELD_DEFAULTS)
-    ccs = models.ManyToManyField(Address, related_name='cc', **KEYFIELD_DEFAULTS)
-    bccs = models.ManyToManyField(Address, related_name='bcc', **KEYFIELD_DEFAULTS)
-    reply_to = models.ForeignKey(Address, related_name='reply_to', **KEYFIELD_DEFAULTS)
+    M2M_KEYFIELD_DEFAULTS = {
+        'blank': True,
+    }
+
+    content = models.ForeignKey(MailBody, on_delete=models.CASCADE,
+                                related_name='mail')
+    signature = models.ForeignKey(Signature, on_delete=models.CASCADE,
+                                  related_name='mail', **KEYFIELD_DEFAULTS)
+    sender = models.ForeignKey(Address, on_delete=models.CASCADE,
+                               related_name='sender', **KEYFIELD_DEFAULTS)
+    recipients = models.ManyToManyField(Address, related_name='recipients',
+                                        **M2M_KEYFIELD_DEFAULTS)
+    ccs = models.ManyToManyField(Address, related_name='cc',
+                                 **M2M_KEYFIELD_DEFAULTS)
+    bccs = models.ManyToManyField(Address, related_name='bcc',
+                                  **M2M_KEYFIELD_DEFAULTS)
+    reply_to = models.ForeignKey(Address, on_delete=models.CASCADE,
+                                 related_name='reply_to', **KEYFIELD_DEFAULTS)
 
     def clone(self):
         """Clone and return a Mail
@@ -144,9 +162,9 @@ class Mail(AbstractNamedModel):
         """
 
         newself = super(Mail, self).clone()
-        newself.recipients = self.recipients.all()
-        newself.ccs = self.ccs.all()
-        newself.bccs = self.bccs.all()
+        newself.recipients.set(self.recipients.all())
+        newself.ccs.set(self.ccs.all())
+        newself.bccs.set(self.bccs.all())
         return newself
 
     @property
@@ -179,7 +197,7 @@ class Mail(AbstractNamedModel):
         "Reply-To may be empty"
 
         if not reply_to and self.reply_to:
-            return text_type(self.reply_to)
+            return str(self.reply_to)
         return reply_to
 
     def get_sender(self, sender=None):
@@ -187,7 +205,7 @@ class Mail(AbstractNamedModel):
 
         if not sender:
             if self.sender:
-                return text_type(self.sender)
+                return str(self.sender)
         else:
             return sender
         raise MailrobotNoSenderError("Mail must have a sender")
@@ -198,7 +216,7 @@ class Mail(AbstractNamedModel):
         attribute = getattr(self, attribute)
         addresses = ()
         if attribute:
-            addresses = set([text_type(row) for row in attribute.all()])
+            addresses = set([str(row) for row in attribute.all()])
             addresses = addresses | set(additional)
         if required and not addresses:
             raise MailrobotNoRecipientsError('No recipient addresses!')
